@@ -1,4 +1,6 @@
-from models import Hierarchy
+from __future__ import unicode_literals
+
+from pagetree.models import Hierarchy
 import warnings
 
 
@@ -52,7 +54,7 @@ def submitted(section, user):
 
 
 def block_submitted(block, user):
-    if user.is_anonymous():
+    if user.is_anonymous:
         # anon can't have submitted a block
         return False
     if hasattr(block, 'needs_submit'):
@@ -61,6 +63,6 @@ def block_submitted(block, user):
                 s = block.unlocked(user)
                 if not s:
                     return False
-            except:
+            except AttributeError:
                 pass
     return True
