@@ -1,3 +1,5 @@
+from __future__ import unicode_literals
+
 from django.core.management.base import BaseCommand
 from django.conf import settings
 from json import loads
@@ -12,26 +14,26 @@ class Command(BaseCommand):
 
     def handle(self, *args, **options):
         if not settings.DEBUG:
-            print "this should never be run on production"
+            print("this should never be run on production")
             return
         if not hasattr(settings, 'PROD_BASE_URL'):
-            print "you must set PROD_BASE_URL"
+            print("you must set PROD_BASE_URL")
             return
-        print "fetching content from prod..."
+        print("fetching content from prod...")
         url = settings.PROD_BASE_URL + "_pagetree/export/"
         if args:
             url = url + "?hierarchy=" + args[0]
         d = loads(GET(url))
-        print "removing old pagetree hierarchy..."
+        print("removing old pagetree hierarchy...")
         Hierarchy.objects.all().delete()
-        print "importing the new one..."
+        print("importing the new one...")
         Hierarchy.from_dict(d)
 
         if not hasattr(settings, 'PROD_MEDIA_BASE_URL'):
-            print "in order to pull down uploaded files,"
-            print "you must set PROD_MEDIA_BASE_URL"
+            print("in order to pull down uploaded files,")
+            print("you must set PROD_MEDIA_BASE_URL")
 
-        print "pulling down uploaded files..."
+        print("pulling down uploaded files...")
         base_len = len(settings.PROD_MEDIA_BASE_URL)
         for upload in d.get('resources', []):
             relative_path = upload[base_len:]
@@ -43,6 +45,6 @@ class Command(BaseCommand):
                 pass
             with open(os.path.join(settings.MEDIA_ROOT,
                                    relative_path), "w") as f:
-                print "  writing %s to %s" % (upload, relative_path)
+                print("  writing %s to %s" % (upload, relative_path))
                 f.write(GET(upload))
-        print "done"
+        print("done")
