@@ -1,35 +1,37 @@
+from __future__ import unicode_literals
+
+import random
+from django.apps import apps
+from django.conf import settings
 from django.contrib.auth.models import User
 from django.core.cache import cache
 from django.db import models
 from django import forms
-from django.template import Context
 from django.template.loader import get_template
 from django.http import Http404
 from django.contrib.contenttypes.models import ContentType
-from django.contrib.contenttypes import generic
-from django.db.models import get_model
+from django.contrib.contenttypes.fields import GenericForeignKey
 from django.template.defaultfilters import slugify
 from json import dumps
 from treebeard.mp_tree import MP_Node
 import django.core.exceptions
-from treebeard.forms import MoveNodeForm
-from pagetree.reports import ReportableInterface, ReportColumnInterface
+from treebeard.forms import movenodeform_factory
 
 
 # dummy it out
 class dummystatsd(object):
     def incr(self, metric):
         pass
+
+
 statsd = dummystatsd()
 
+
 try:
     from django_statsd.clients import statsd
 except ImportError:
     pass
 
-settings = None
-from django.conf import settings
-
 
 class Hierarchy(models.Model):
     name = models.CharField(max_length=256)
@@ -43,7 +45,7 @@ class Hierarchy(models.Model):
     def get_absolute_url(self):
         return self.base_url
 
-    def __unicode__(self):
+    def __str__(self):
         return self.name
 
     def get_root(self):
@@ -80,9 +82,14 @@ class Hierarchy(models.Model):
             raise Http404()
         return s
 
-    def available_pageblocks(self):
-        if hasattr(settings, 'PAGEBLOCKS') and settings.PAGEBLOCKS is not None:
-            return [get_model(*pb.split('.')) for pb in settings.PAGEBLOCKS]
+    @staticmethod
+    def available_pageblocks():
+        if hasattr(settings, 'PAGEBLOCKS') and \
+           settings.PAGEBLOCKS is not None:
+            models = [
+                apps.get_model(*pb.split('.'))
+                for pb in settings.PAGEBLOCKS]
+            return models
         else:
             return []
 
@@ -109,8 +116,8 @@ class Hierarchy(models.Model):
             s.add_child_section_from_dict(c)
         return s
 
-    @classmethod
-    def from_dict(cls, d):
+    @staticmethod
+    def from_dict(d):
         h = Hierarchy.objects.create(name=d.get('name', ''),
                                      base_url=d.get('base_url', '/'))
         for s in d.get('sections', []):
@@ -118,7 +125,7 @@ class Hierarchy(models.Model):
         return h
 
     def get_user_location(self, user):
-        if user.is_anonymous():
+        if user.is_anonymous:
             return "/"
         (ul, created) = UserLocation.objects.get_or_create(
             user=user,
@@ -137,11 +144,18 @@ class Hierarchy(models.Model):
         ul.path = path
         ul.save()
 
+    @staticmethod
+    def clone(original, name, base_url):
+        hierarchy = Hierarchy.objects.create(name=name, base_url=base_url)
+        root = Section.add_root(label="Root", slug="", hierarchy=hierarchy)
+        Section.clone(original.get_root(), root)
+        return hierarchy
+
 
 class Section(MP_Node):
     label = models.CharField(max_length=256)
     slug = models.SlugField()
-    hierarchy = models.ForeignKey(Hierarchy)
+    hierarchy = models.ForeignKey(Hierarchy, on_delete=models.CASCADE)
     show_toc = models.BooleanField(
         default=False,
         help_text=("list table of contents of "
@@ -178,6 +192,11 @@ class Section(MP_Node):
         cache.delete("pagetree.%d.get_absolute_url" % self.id)
         cache.delete("pagetree.%d.is_last_child" % self.id)
         cache.delete("pagetree.%d.get_edit_url" % self.id)
+        cache.delete("pagetree.%d.get_tree_depth" % self.id)
+
+        if hasattr(settings, 'PAGETREE_CUSTOM_CACHE_CLEAR') and \
+           callable(settings.PAGETREE_CUSTOM_CACHE_CLEAR):
+            settings.PAGETREE_CUSTOM_CACHE_CLEAR(self)
 
     def clear_tree_cache(self):
         depth_first_traversal = self.get_root().get_annotated_list()
@@ -196,13 +215,16 @@ class Section(MP_Node):
             return self
         return self.get_ancestors()[1]
 
+    def get_descendants(self):
+        return MP_Node.get_descendants(self).select_related('hierarchy')
+
     def is_first_child(self):
         return self.get_first_sibling().id == self.id
 
     def is_last_child(self):
         key = "pagetree.%d.is_last_child" % self.id
         v = cache.get(key)
-        if v:
+        if v is not None:
             statsd.incr("pagetree.is_last_child.cache_hit")
             return v
         statsd.incr("pagetree.is_last_child.cache_miss")
@@ -266,13 +288,13 @@ class Section(MP_Node):
                                         css_extra=css_extra,
                                         content_object=content_object)
 
-    def __unicode__(self):
+    def __str__(self):
         return self.label
 
     def get_absolute_url(self):
         key = "pagetree.%d.get_absolute_url" % self.id
         v = cache.get(key)
-        if v:
+        if v is not None:
             statsd.incr("pagetree.get_absolute_url.cache_hit")
             return v
         statsd.incr("pagetree.get_absolute_url.cache_miss")
@@ -285,14 +307,14 @@ class Section(MP_Node):
         slugs = [a.slug for a in ancestors[1:]]
         if len(slugs) == 0:
             return self.hierarchy.get_absolute_url() + self.slug + "/"
-        url = (self.hierarchy.get_absolute_url() + "/".join(slugs)
-               + "/" + self.slug + "/")
+        url = (self.hierarchy.get_absolute_url() + "/".join(slugs) +
+               "/" + self.slug + "/")
         return url
 
     def get_edit_url(self):
         key = "pagetree.%d.get_edit_url" % self.id
         v = cache.get(key)
-        if v:
+        if v is not None:
             statsd.incr("pagetree.get_edit_url.cache_hit")
             return v
         statsd.incr("pagetree.get_edit_url.cache_miss")
@@ -306,8 +328,8 @@ class Section(MP_Node):
         if len(slugs) == 0:
             return (
                 self.hierarchy.get_absolute_url() + "edit/" + self.slug + "/")
-        url = (self.hierarchy.get_absolute_url() + "edit/" + "/".join(slugs)
-               + "/" + self.slug + "/")
+        url = (self.hierarchy.get_absolute_url() + "edit/" + "/".join(slugs) +
+               "/" + self.slug + "/")
         return url
 
     def get_instructor_url(self):
@@ -338,11 +360,13 @@ class Section(MP_Node):
             slug = forms.CharField(initial=self.slug)
             show_toc = forms.BooleanField(
                 initial=self.show_toc,
+                required=False,
                 label="Show Table of Contents",
                 help_text=("list table of contents of "
                            "immediate child sections (if applicable)"))
             deep_toc = forms.BooleanField(
                 initial=self.deep_toc,
+                required=False,
                 label="Show Deep Table of Contents",
                 help_text=(
                     "include children of children (etc) in TOC. "
@@ -350,7 +374,10 @@ class Section(MP_Node):
         return EditSectionForm()
 
     def move_form(self):
-        return MoveNodeForm(instance=self)
+        MoveSectionForm = movenodeform_factory(
+            type(self),
+            exclude=('label', 'slug', 'hierarchy', 'show_toc', 'deep_toc'))
+        return MoveSectionForm(instance=self)
 
     def update_children_order(self, children_ids):
         """children_ids is a list of Section ids for the children
@@ -382,10 +409,24 @@ class Section(MP_Node):
         return self.hierarchy.available_pageblocks()
 
     def add_pageblock_form(self):
-        class EditForm(forms.Form):
-            label = forms.CharField()
-            css_extra = forms.CharField(label="extra CSS classes")
-        return EditForm()
+        # This unique id should instead be derived from the block type's
+        # ID, but that requires a new templatetag. This works for now.
+        unique_id = random.randint(0, 10000)
+
+        class AddPageBlockForm(forms.Form):
+            label = forms.CharField(
+                required=False,
+                widget=forms.TextInput(
+                    attrs={'id': 'id_label_%d' % unique_id}
+                ))
+            css_extra = forms.CharField(
+                label='extra CSS classes',
+                required=False,
+                widget=forms.TextInput(
+                    attrs={'id': 'id_css_extra_%d' % unique_id}
+                ))
+
+        return AddPageBlockForm()
 
     def get_first_leaf(self):
         if self.is_leaf():
@@ -400,16 +441,18 @@ class Section(MP_Node):
     def reset(self, user):
         """ clear a user's responses to all pageblocks on this page """
         for p in self.pageblock_set.all():
-            if hasattr(p.block(), 'needs_submit'):
-                if p.block().needs_submit():
-                    p.block().clear_user_submissions(user)
+            block = p.block()
+            if hasattr(block, 'needs_submit'):
+                if block.needs_submit():
+                    block.clear_user_submissions(user)
 
     def submit(self, request_data, user):
         """ store users's responses to the pageblocks on this page """
         proceed = True
         for p in self.pageblock_set.all():
-            if hasattr(p.block(), 'needs_submit'):
-                if p.block().needs_submit():
+            block = p.block()
+            if hasattr(block, 'needs_submit'):
+                if block.needs_submit():
                     prefix = "pageblock-%d-" % p.id
                     data = dict()
                     for k in request_data.keys():
@@ -420,33 +463,35 @@ class Section(MP_Node):
                                 data[k[len(prefix):]] = request_data[k]
                             else:
                                 data[k[len(prefix):]] = v
-                    p.block().submit(user, data)
-                    if hasattr(p.block(), 'redirect_to_self_on_submit'):
+                    block.submit(user, data)
+                    if hasattr(block, 'redirect_to_self_on_submit'):
                         # semi bug here?
                         # proceed will only be set by the last submittable
                         # block on the page. previous ones get ignored.
-                        proceed = not p.block().redirect_to_self_on_submit()
+                        proceed = not block.redirect_to_self_on_submit()
         return proceed
 
     def needs_submit(self):
         """ if any blocks on the page need to be submitted """
         for p in self.pageblock_set.all():
-            if hasattr(p.block(), 'needs_submit'):
-                if p.block().needs_submit():
+            block = p.block()
+            if hasattr(block, 'needs_submit'):
+                if block.needs_submit():
                     return True
         return False
 
     def allow_redo(self, **kwargs):
         """ if any blocks on the page are allowed to be resubmitted """
         for p in self.pageblock_set.all():
-            if hasattr(p.block(), 'allow_redo'):
-                if hasattr(p.block().allow_redo, '__call__'):
-                    if p.block().allow_redo(**kwargs):
+            block = p.block()
+            if hasattr(block, 'allow_redo'):
+                if hasattr(block.allow_redo, '__call__'):
+                    if block.allow_redo(**kwargs):
                         return True
                 else:
                     # not callable, so we expect it
                     # to just be a boolean property
-                    if p.block().allow_redo:
+                    if block.allow_redo:
                         return True
         return False
 
@@ -454,20 +499,32 @@ class Section(MP_Node):
         """ if all blocks on the page that require submissions
         have been submitted """
         for p in self.pageblock_set.all():
-            if hasattr(p.block(), 'needs_submit'):
-                if p.block().needs_submit():
+            block = p.block()
+            if hasattr(block, 'needs_submit'):
+                if block.needs_submit():
                     try:
-                        s = p.block().unlocked(user)
+                        s = block.unlocked(user)
                         if not s:
                             # there's an unsubmitted block
                             return False
-                    except:
+                    except (AttributeError, ValueError):
                         # most likely: no unlocked() method
                         pass
         # made it all the way through without any blocks
         # reporting that they are unsubmitted
         return True
 
+    def unlocked(self, user):
+        """Returns True if all the blocks on this page are unlocked."""
+
+        for p in self.pageblock_set.all():
+            block = p.block()
+            if hasattr(block, 'unlocked'):
+                if not block.unlocked(user):
+                    return False
+
+        return True
+
     def as_dict(self):
         return dict(
             label=self.label,
@@ -490,18 +547,32 @@ class Section(MP_Node):
             self.add_child_section_from_dict(c)
 
     def add_pageblock_from_dict(self, d):
-        blocktype = d.get('block_type', '')
+        target_type = d.get('block_type', '')
+
         # now we need to figure out which kind of pageblock to create
+        found_pbclass = None
         for pb_class in self.available_pageblocks():
             if not pb_class:
                 continue
-            if pb_class.display_name == blocktype:
+            if pb_class.display_name == target_type:
                 # a match
-                if hasattr(pb_class, 'create_from_dict'):
-                    block = pb_class.create_from_dict(d)
-                    self.append_pageblock(label=d.get('label', ''),
-                                          css_extra=d.get('css_extra', ''),
-                                          content_object=block)
+                found_pbclass = pb_class
+                break
+
+        if found_pbclass:
+            if hasattr(found_pbclass, 'create_from_dict'):
+                # Prepare a dictionary for the custom pageblock's
+                # create_from_dict method by removing the data that's
+                # only used elsewhere.
+                block_dict = d.copy()
+                block_dict.pop('block_type', None)
+                block_dict.pop('label', None)
+                block_dict.pop('css_extra', None)
+                block = found_pbclass.create_from_dict(block_dict)
+
+                self.append_pageblock(label=d.get('label', ''),
+                                      css_extra=d.get('css_extra', ''),
+                                      content_object=block)
 
     def add_child_section_from_dict(self, d):
         s = self.append_child(
@@ -523,16 +594,14 @@ class Section(MP_Node):
                 section=self,
                 user=user)
         except django.core.exceptions.MultipleObjectsReturned:
-            upv = UserPageVisit.objects.filter(section=self, user=user)[0]
+            upv = UserPageVisit.objects.filter(
+                section=self, user=user).first()
         upv.status = status
         upv.save()
 
     def get_uservisit(self, user):
-        r = self.userpagevisit_set.filter(user=user)
-        if r.count() > 0:
-            return r[0]
-        else:
-            return None
+        # returns None if the userpagevisit doesn't exist
+        return self.userpagevisit_set.filter(user=user).first()
 
     def save_version(self, user, activity=None, comment=None):
         if not activity:
@@ -546,8 +615,8 @@ class Section(MP_Node):
             data=json,
             activity=activity,
             comment=comment)
-        return v
         self.clear_tree_cache()
+        return v
 
     def gate_check(self, user):
         """ return bool, section tuple for whether the user
@@ -582,23 +651,51 @@ class Section(MP_Node):
         # the current section?!
         assert False, "current section not found in traversal"
 
+    @staticmethod
+    def clone(original, section):
+        for b in original.pageblock_set.all():
+            section.add_pageblock_from_dict(b.as_dict())
+        section.save()
+
+        children = original.get_children()
+        for old_child in children:
+            child = section.append_child(label=old_child.label,
+                                         slug=old_child.slug,
+                                         show_toc=old_child.show_toc,
+                                         deep_toc=old_child.deep_toc)
+
+            Section.clone(old_child, child)
+
+        return section
+
+    def get_tree_depth(self):
+        key = 'pagetree.{}.get_tree_depth'.format(self.pk)
+        v = cache.get(key)
+        if v is not None:
+            return v
+
+        for idx, sec in enumerate(self.get_tree()):
+            if sec == self:
+                cache.set(key, idx)
+                return idx
+
 
 class PageBlock(models.Model):
-    section = models.ForeignKey(Section)
+    section = models.ForeignKey(Section, on_delete=models.CASCADE)
     ordinality = models.PositiveIntegerField(default=1)
     label = models.CharField(max_length=256, blank=True, null=True)
     css_extra = models.CharField(
         max_length=256, blank=True, null=True,
         help_text="extra CSS classes (space separated)")
 
-    content_type = models.ForeignKey(ContentType)
+    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
     object_id = models.PositiveIntegerField()
-    content_object = generic.GenericForeignKey('content_type', 'object_id')
+    content_object = GenericForeignKey('content_type', 'object_id')
 
     class Meta:
         ordering = ('section', 'ordinality',)
 
-    def __unicode__(self):
+    def __str__(self):
         return "%s [%d]: %s" % (self.section.label, self.ordinality,
                                 self.label)
 
@@ -608,18 +705,19 @@ class PageBlock(models.Model):
     def edit_label(self):
         """ provide a label for the pageblock to make the
         edit interface easier to read """
-        if hasattr(self.block(), 'edit_label'):
-            return self.block().edit_label()
+        block = self.block()
+        if hasattr(block, 'edit_label'):
+            return block.edit_label()
         else:
-            return self.block().display_name
+            return block.display_name
 
     def render(self, **kwargs):
         if hasattr(self.content_object, "template_file"):
             t = get_template(getattr(self.content_object, "template_file"))
             d = kwargs
-            d['block'] = self.content_object
-            c = Context(d)
-            return t.render(c)
+            d['pageblock'] = self.content_object
+            d['block'] = d['pageblock']
+            return t.render(d)
         else:
             return self.content_object.render()
 
@@ -628,8 +726,7 @@ class PageBlock(models.Model):
             t = get_template(getattr(self.content_object, "js_template_file"))
             d = kwargs
             d['block'] = self.content_object
-            c = Context(d)
-            return t.render(c)
+            return t.render(d)
         elif hasattr(self.content_object, "js_render"):
             return self.content_object.js_render()
         else:
@@ -640,8 +737,7 @@ class PageBlock(models.Model):
             t = get_template(getattr(self.content_object, "css_template_file"))
             d = kwargs
             d['block'] = self.content_object
-            c = Context(d)
-            return t.render(c)
+            return t.render(d)
         elif hasattr(self.content_object, "css_render"):
             return self.content_object.css_render()
         else:
@@ -653,8 +749,7 @@ class PageBlock(models.Model):
                                      "summary_template_file"))
             d = kwargs
             d['block'] = self.content_object
-            c = Context(d)
-            return t.render(c)
+            return t.render(d)
         elif hasattr(self.content_object, "summary_render"):
             return self.content_object.summary_render()
         else:
@@ -662,9 +757,19 @@ class PageBlock(models.Model):
 
     def default_edit_form(self):
         class EditForm(forms.Form):
-            label = forms.CharField(initial=self.label)
-            css_extra = forms.CharField(initial=self.css_extra,
-                                        label="extra CSS classes")
+            label = forms.CharField(
+                initial=self.label,
+                required=False,
+                widget=forms.TextInput(
+                    attrs={'id': 'id_label_%d' % self.pk}
+                ))
+            css_extra = forms.CharField(
+                initial=self.css_extra,
+                label='extra CSS classes',
+                required=False,
+                widget=forms.TextInput(
+                    attrs={'id': 'id_css_extra_%d' % self.pk}
+                ))
         return EditForm()
 
     def edit_form(self):
@@ -690,11 +795,23 @@ class PageBlock(models.Model):
         d['block_type'] = self.content_object.display_name
         return d
 
+    def import_from_dict(self, d):
+        self.label = d.get('label', '')
+        self.css_extra = d.get('css_extra', '')
+        self.save()
+
+        if hasattr(self.content_object, 'import_from_dict'):
+            self.content_object.import_from_dict(d)
+
+    @classmethod
+    def create_from_dict(cls, d):
+        return cls.objects.create(**d)
+
 
 class UserLocation(models.Model):
     """ last path a given user visited (for a particular hierarchy) """
-    user = models.ForeignKey(User)
-    hierarchy = models.ForeignKey(Hierarchy)
+    user = models.ForeignKey(User, on_delete=models.CASCADE)
+    hierarchy = models.ForeignKey(Hierarchy, on_delete=models.CASCADE)
     path = models.CharField(max_length=256, default="/")
 
     class Meta:
@@ -703,8 +820,8 @@ class UserLocation(models.Model):
 
 class UserPageVisit(models.Model):
     """ for detailed tracking """
-    user = models.ForeignKey(User)
-    section = models.ForeignKey(Section)
+    user = models.ForeignKey(User, on_delete=models.CASCADE)
+    section = models.ForeignKey(Section, on_delete=models.CASCADE)
     status = models.CharField(max_length=256, default="incomplete")
     first_visit = models.DateTimeField(auto_now_add=True)
     last_visit = models.DateTimeField(auto_now=True)
@@ -715,8 +832,8 @@ class UserPageVisit(models.Model):
 
 class Version(models.Model):
     """ very basic versioning """
-    section = models.ForeignKey(Section)
-    user = models.ForeignKey(User)
+    section = models.ForeignKey(Section, on_delete=models.CASCADE)
+    user = models.ForeignKey(User, on_delete=models.CASCADE)
     saved_at = models.DateTimeField(auto_now_add=True)
     activity = models.TextField(default="", blank=True, null=True)
     data = models.TextField(default="", blank=True, null=True)
@@ -725,6 +842,9 @@ class Version(models.Model):
     class Meta:
         ordering = ["-saved_at", ]
 
+    def __lt__(self, other):
+        return self.saved_at < other.saved_at
+
     def more_recent_versions(self):
         versions = list(
             self.section.version_set.filter(
@@ -732,81 +852,5 @@ class Version(models.Model):
         for s in self.section.get_descendants():
             for v in s.version_set.filter(saved_at__gt=self.saved_at):
                 versions.append(v)
-        versions.sort(lambda a, b: cmp(b.saved_at, a.saved_at))
+        versions.sort()
         return versions
-
-
-class TestReportColumn(ReportColumnInterface):
-
-    def identifier(self):
-        return "Test Report Column"
-
-    def metadata(self):
-        return ['', self.identifier(), "generic",
-                "string", "this is a test"]
-
-    def user_value(self, user):
-        return user.username
-
-
-class TestBlock(models.Model):
-    """ this is a pageblock that is exclusively for pagetree's
-    internal tests so we have some kind of block to test with
-    without having to pull in the whole django-pageblocks
-    package and its dependencies.
-
-    You should never use it. In fact, forget that you ever saw this.
-    """
-    pageblocks = generic.GenericRelation(PageBlock)
-    body = models.TextField(blank=True)
-
-    template_file = "pagetree/testblock.html"
-    display_name = "Test Block"
-
-    def __unicode__(self):
-        return unicode(self.pageblock())
-
-    def pageblock(self):
-        return self.pageblocks.all()[0]
-
-    @classmethod
-    def add_form(self):
-        class AddForm(forms.Form):
-            body = forms.CharField(
-                widget=forms.widgets.Textarea(attrs={'cols': 80}))
-        return AddForm()
-
-    @classmethod
-    def create(self, request):
-        return TestBlock.objects.create(body=request.POST.get('body', ''))
-
-    @classmethod
-    def create_from_dict(self, d):
-        return TestBlock.objects.create(body=d.get('body', ''))
-
-    def edit_form(self):
-        class EditForm(forms.Form):
-            body = forms.CharField(widget=forms.widgets.Textarea(),
-                                   initial=self.body)
-        return EditForm()
-
-    def edit(self, vals, files):
-        self.body = vals.get('body', '')
-        self.save()
-
-    def as_dict(self):
-        return dict(body=self.body)
-
-    def summary_render(self):
-        if len(self.body) < 61:
-            return self.body
-        else:
-            return self.body[:61] + "..."
-
-    def report_metadata(self):
-        return [TestReportColumn()]
-
-    def report_values(self):
-        return [TestReportColumn()]
-
-ReportableInterface.register(TestBlock)
