@@ -1,4 +1,8 @@
+from __future__ import unicode_literals
+
 from django import template
+from django.utils.encoding import smart_text
+
 register = template.Library()
 
 
@@ -28,3 +32,14 @@ def get_user_section_status(parser, token):
         # handle "as some_var"
         var_name = token.split_contents()[1:][3]
     return GetUserSectionStatus(user, section, var_name)
+
+
+@register.simple_tag
+def is_section_unlocked(section, user):
+    if hasattr(section, 'unlocked'):
+        is_unlocked = section.unlocked(user)
+    else:
+        is_unlocked = True
+
+    # Convert to 1 or 0 to parse it easily in the template.
+    return smart_text(int(is_unlocked))
