@@ -1,9 +1,42 @@
+from __future__ import unicode_literals
+
 from django.test import TestCase
 from django.contrib.auth.models import User
-from django.utils import unittest
 from django.http import Http404
-from pagetree.models import Hierarchy, PageBlock, UserPageVisit
+from django.utils.encoding import smart_text
 import django.db
+from django.db import transaction
+
+from pagetree.models import Hierarchy, PageBlock, UserPageVisit
+from pagetree.test_models import TestBlock
+from pagetree.tests.factories import (
+    RootSectionFactory, TestBlockFactory, UserFactory, UserPageVisitFactory,
+    HierarchyFactory
+)
+
+
+class UserTest(TestCase):
+    def setUp(self):
+        self.u = UserFactory()
+
+    def test_is_valid_from_factory(self):
+        self.u.full_clean()
+
+
+class HierarchyTest(TestCase):
+    def setUp(self):
+        self.h = HierarchyFactory()
+
+    def test_is_valid_from_factory(self):
+        self.h.full_clean()
+
+
+class RootSectionTest(TestCase):
+    def setUp(self):
+        self.s = RootSectionFactory()
+
+    def test_is_valid_from_factory(self):
+        self.s.full_clean()
 
 
 class EmptyHierarchyTest(TestCase):
@@ -67,13 +100,8 @@ class EmptyHierarchyTest(TestCase):
             self.h.id,
             Hierarchy.get_hierarchy("main").id)
 
-    def test_unicode(self):
-        self.assertEqual(
-            str(self.h),
-            "main")
-        self.assertEqual(
-            unicode(self.h),
-            "main")
+    def test_str(self):
+        self.assertEqual(smart_text(self.h), "main")
 
     def test_empty_top_level(self):
         self.assertEqual(
@@ -104,7 +132,7 @@ class EmptyHierarchyTest(TestCase):
         self.assertEqual(s.slug, '')
 
 
-class OneLevelDeepTest(unittest.TestCase):
+class OneLevelDeepTest(TestCase):
     def setUp(self):
         self.h = Hierarchy.objects.create(name="main", base_url="")
         self.root = self.h.get_root()
@@ -175,6 +203,8 @@ class OneLevelDeepTest(unittest.TestCase):
         self.assertEqual(
             'css_extra' in f.fields,
             True)
+        self.assertFalse(f.fields['label'].required)
+        self.assertFalse(f.fields['css_extra'].required)
 
     def test_valid_path(self):
         self.assertEqual(self.h.find_section_from_path("section-1/"),
@@ -253,9 +283,9 @@ class OneLevelDeepTest(unittest.TestCase):
             self.root.get_previous(),
             None)
 
-    def test_section_unicode(self):
+    def test_section_str(self):
         self.assertEqual(
-            str(self.section1),
+            smart_text(self.section1),
             "Section 1")
 
     def test_section_get_absolute_url(self):
@@ -299,8 +329,13 @@ class OneLevelDeepTest(unittest.TestCase):
         r = self.root.get_children()
         self.assertEqual([s.id for s in r], normal_order)
 
+    def test_get_tree_depth(self):
+        self.assertEqual(self.section1.get_tree_depth(), 1)
+        self.assertEqual(self.section2.get_tree_depth(), 2)
+        self.assertEqual(self.section3.get_tree_depth(), 3)
 
-class OneLevelWithBlocksTest(unittest.TestCase):
+
+class OneLevelWithBlocksTest(TestCase):
     def setUp(self):
         self.h = Hierarchy.objects.create(name="main", base_url="")
         self.root = self.h.get_root()
@@ -328,10 +363,10 @@ class OneLevelWithBlocksTest(unittest.TestCase):
     def tearDown(self):
         self.h.delete()
 
-    def test_unicode(self):
+    def test_str(self):
         b = self.section1.pageblock_set.all()[0]
         self.assertEqual(
-            str(b),
+            smart_text(b),
             "Section 1 [1]: ")
 
     def test_edit_label(self):
@@ -379,6 +414,8 @@ class OneLevelWithBlocksTest(unittest.TestCase):
         self.assertEqual(
             "css_extra" in f.fields,
             True)
+        self.assertFalse(f.fields['label'].required)
+        self.assertFalse(f.fields['css_extra'].required)
 
     def test_edit(self):
         b = self.section1.pageblock_set.all()[0]
@@ -419,8 +456,32 @@ class OneLevelWithBlocksTest(unittest.TestCase):
             [p.id for p in self.section1.pageblock_set.all()],
             normal_order)
 
+    def test_import_from_dict(self):
+        b = self.section1.pageblock_set.first()
+        d = {'label': 'new label',
+             'css_extra': 'new css_extra',
+             'body': 'new body'}
+        b.import_from_dict(d)
+        self.assertEquals(b.label, 'new label')
+        self.assertEquals(b.css_extra, 'new css_extra')
 
-class UserTrackingTest(unittest.TestCase):
+        sub = b.block()
+        self.assertEquals(sub.body, 'new body')
+
+    def test_import_custom_block_dict(self):
+        b = self.section1.pageblock_set.first()
+        d = {
+            'block_type': 'Test Block',
+            'body': 'abc',
+        }
+        b.import_from_dict(d)
+
+        sub = b.block()
+        self.assertEquals(sub.display_name, 'Test Block')
+        self.assertEquals(sub.body, 'abc')
+
+
+class UserTrackingTest(TestCase):
     def setUp(self):
         self.h = Hierarchy.objects.create(name="main", base_url="")
         self.root = self.h.get_root()
@@ -514,12 +575,14 @@ class UserTrackingTest(unittest.TestCase):
     def test_user_pagevisit_multiple(self):
         self.section1.user_pagevisit(self.user, status="complete")
         try:
-            # then stuff another one in manually to simulate a race condition
-            UserPageVisit.objects.create(section=self.section1,
-                                         user=self.user,
-                                         status="bad status")
-            # should not be able to happen
-            self.assertEqual(True, False)
+            with transaction.atomic():
+                # then stuff another one in manually to simulate a
+                # race condition
+                UserPageVisit.objects.create(section=self.section1,
+                                             user=self.user,
+                                             status="bad status")
+                # should not be able to happen
+                self.assertEqual(True, False)
         except django.db.IntegrityError:
             self.assertEqual(True, True)
         self.section1.user_pagevisit(self.user, status="complete")
@@ -545,7 +608,7 @@ class UserTrackingTest(unittest.TestCase):
             children[1].gate_check(self.user), (False, children[0]))
 
 
-class VersionTest(unittest.TestCase):
+class VersionTest(TestCase):
     def setUp(self):
         self.h = Hierarchy.objects.create(name="main", base_url="")
         self.root = self.h.get_root()
@@ -646,7 +709,7 @@ class VersionTest(unittest.TestCase):
             1)
 
 
-class MultipleLevelsTest(unittest.TestCase):
+class MultipleLevelsTest(TestCase):
     def setUp(self):
         self.h = Hierarchy.objects.create(name="main", base_url="")
         self.root = self.h.get_root()
@@ -693,7 +756,7 @@ class MultipleLevelsTest(unittest.TestCase):
         self.section5 = self.section4.get_children()[0]
 
     def tearDown(self):
-        self.h.delete()
+        Hierarchy.objects.all().delete()
 
     def test_get_absolute_url(self):
         self.assertEqual(self.section4.get_absolute_url(),
@@ -710,3 +773,77 @@ class MultipleLevelsTest(unittest.TestCase):
     def test_get_edit_url_two_down(self):
         self.assertEqual(self.section5.get_edit_url(),
                          "edit/section-3/section-4/section-5/")
+
+    def test_clone_hierarchy(self):
+        self.section1.add_pageblock_from_dict({
+            'block_type': 'Test Block',
+            'body': 'test body',
+        })
+        duplicate = Hierarchy.clone(self.h, 'foo', '/foo/')
+
+        descendants = duplicate.get_root().get_descendants()
+        self.assertTrue(descendants[0].label, 'Section 1')
+        self.assertTrue(descendants[0].depth, 1)
+        self.assertEquals(descendants[0].pageblock_set.count(), 1)
+
+        self.assertTrue(descendants[1].label, 'Section 2')
+        self.assertTrue(descendants[1].depth, 1)
+
+        self.assertTrue(descendants[2].label, 'Section 3')
+        self.assertTrue(descendants[2].depth, 1)
+
+        self.assertTrue(descendants[3].label, 'Section 4')
+        self.assertTrue(descendants[3].depth, 2)
+
+        self.assertTrue(descendants[4].label, 'Section 5')
+        self.assertTrue(descendants[4].depth, 3)
+
+
+class SectionTest(TestCase):
+    def setUp(self):
+        self.user = UserFactory()
+        self.section = RootSectionFactory()
+
+    def test_add_pageblock_from_dict(self):
+        self.section.add_pageblock_from_dict({
+            'block_type': 'Test Block',
+            'body': 'test body',
+        })
+        block = self.section.pageblock_set.first()
+        assert(block is not None)
+        self.assertEqual(block.block().body, 'test body')
+        self.assertEqual(
+            self.section.pageblock_set.first(), block,
+            'The PageBlock has been added to the Section')
+
+    def test_empty_section_is_unlocked(self):
+        self.assertTrue(self.section.unlocked(self.user))
+
+    def test_section_with_test_block_is_unlocked(self):
+        self.section.add_pageblock_from_dict({'block_type': 'Test Block'})
+        self.assertEqual(self.section.pageblock_set.count(), 1)
+
+        # TestBlock doesn't have an "unlocked" method defined, so this
+        # tests that Section.unlocked() handles that case.
+        self.assertTrue(self.section.unlocked(self.user))
+
+
+class TestBlockTest(TestCase):
+    def setUp(self):
+        self.b = TestBlockFactory()
+
+    def test_is_valid_from_factory(self):
+        self.b.full_clean()
+
+    def test_create_from_dict(self):
+        testblock = TestBlock.create_from_dict({
+            'body': 'abc',
+        })
+        self.assertEqual(testblock.display_name, 'Test Block')
+        self.assertEqual(testblock.body, 'abc')
+
+
+class UserPageVisitTest(TestCase):
+    def test_is_valid_from_factory(self):
+        upv = UserPageVisitFactory()
+        upv.full_clean()
