@@ -1,12 +1,14 @@
-from annoying.decorators import render_to
+from __future__ import unicode_literals
+
 from django.http import HttpResponse, HttpResponseRedirect
-from django.shortcuts import get_object_or_404, render_to_response
-from django.template.context import RequestContext
+from django.utils.encoding import smart_text
+from django.shortcuts import get_object_or_404, render
 from django.template.defaultfilters import slugify
+from django.template.response import TemplateResponse
 from json import dumps, loads
 from pagetree.helpers import get_section_from_path
+from pagetree.forms import MoveSectionForm
 from pagetree.models import Section, PageBlock, Hierarchy, Version
-from treebeard.forms import MoveNodeForm
 
 
 def reorder_pageblocks(request, section_id, id_prefix="pageblock_id_"):
@@ -14,7 +16,7 @@ def reorder_pageblocks(request, section_id, id_prefix="pageblock_id_"):
         return HttpResponse("only use POST for this")
     section = get_object_or_404(Section, id=section_id)
     section.save_version(request.user, activity="reorder pageblocks")
-    keys = request.GET.keys()
+    keys = list(request.GET)
     keys.sort(key=lambda x: int(x.split('_')[-1]))
     pageblocks = [int(request.GET[k]) for k in keys if k.startswith(id_prefix)]
     section.update_pageblocks_order(pageblocks)
@@ -26,7 +28,7 @@ def reorder_section_children(request, section_id, id_prefix="section_id_"):
         return HttpResponse("only use POST for this")
     section = get_object_or_404(Section, id=section_id)
     section.save_version(request.user, activity="reorder children")
-    keys = request.GET.keys()
+    keys = list(request.GET)
     keys.sort(key=lambda x: int(x.split('_')[-1]))
     children = [int(request.GET[k]) for k in keys if k.startswith(id_prefix)]
     section.update_children_order(children)
@@ -36,28 +38,30 @@ def reorder_section_children(request, section_id, id_prefix="section_id_"):
 
 def delete_pageblock(request, pageblock_id, success_url=None):
     block = get_object_or_404(PageBlock, id=pageblock_id)
-    section = block.section
-    section.save_version(request.user,
-                         activity="delete block [%s]" % unicode(block))
-    try:
-        block.block().delete()
-    except AttributeError:
-        # if the model has been refactored, we sometimes
-        # end up with 'stub' pageblocks floating around
-        # that no longer have a block object associated
-        # it's nice to still be able to delete them
-        # without having to scrap the whole db and start over
-        pass
-    block.delete()
-    if success_url is None:
-        success_url = section.get_edit_url()
-    return HttpResponseRedirect(success_url)
+    if request.method == "POST":
+        section = block.section
+        section.save_version(request.user,
+                             activity="delete block [%s]" % smart_text(block))
+        try:
+            block.block().delete()
+        except AttributeError:
+            # if the model has been refactored, we sometimes
+            # end up with 'stub' pageblocks floating around
+            # that no longer have a block object associated
+            # it's nice to still be able to delete them
+            # without having to scrap the whole db and start over
+            pass
+        block.delete()
+        if success_url is None:
+            success_url = section.get_edit_url()
+        return HttpResponseRedirect(success_url)
+    return TemplateResponse(request, 'delete_confirm.html')
 
 
 def export_pageblock_json(request, pageblock_id):
     block = get_object_or_404(PageBlock, id=pageblock_id)
-    json = dumps(block.block().as_dict())
-    r = HttpResponse(json, content_type="application/json")
+    the_json = dumps(block.as_dict())
+    r = HttpResponse(the_json, content_type="application/json")
     r['Content-Disposition'] = ('attachment; filename=pageblock_%d.json'
                                 % int(pageblock_id))
     return r
@@ -67,16 +71,15 @@ def import_pageblock_json(request, pageblock_id):
     block = get_object_or_404(PageBlock, id=pageblock_id)
     block.section.save_version(
         request.user,
-        activity="importing pageblock json [%s]" % unicode(block))
+        activity="importing pageblock json [%s]" % smart_text(block))
     if request.method == "POST":
         if 'file' not in request.FILES:
             return HttpResponse("you must upload a json file")
         json = loads(request.FILES['file'].read())
-        block.block().import_from_dict(json)
+        block.import_from_dict(json)
         return HttpResponseRedirect(block.section.get_edit_url())
     else:
-        return render_to_response("import_json.html", dict(),
-                                  context_instance=RequestContext(request))
+        return render(request, "import_json.html")
 
 
 def edit_pageblock(request, pageblock_id, success_url=None):
@@ -84,7 +87,7 @@ def edit_pageblock(request, pageblock_id, success_url=None):
     section = block.section
     section.save_version(
         request.user,
-        activity="edit pageblock [%s]" % unicode(block))
+        activity="edit pageblock [%s]" % smart_text(block))
     block.edit(request.POST, request.FILES)
     if success_url is None:
         success_url = section.get_edit_url()
@@ -96,8 +99,13 @@ def edit_section(request, section_id, success_url=None):
     section.save_version(request.user, activity="edit section")
     section.label = request.POST.get('label', '')
     section.slug = slugify(request.POST.get('slug', section.label))[:50]
-    section.show_toc = request.POST.get('show_toc', False)
-    section.deep_toc = request.POST.get('deep_toc', False)
+
+    value = request.POST.get('show_toc', False)
+    section.show_toc = value in ('True', True, 'on')
+
+    value = request.POST.get('deep_toc', False)
+    section.deep_toc = value in ('True', True, 'on')
+
     section.save()
     section.enforce_slug()
     section.save()
@@ -114,7 +122,7 @@ def delete_section(request, section_id, success_url=None):
         if parent:
             parent.save_version(
                 request.user,
-                activity="delete child section [%s]" % unicode(section))
+                activity="delete child section [%s]" % smart_text(section))
         section.delete()
         if success_url is None:
             if parent:
@@ -123,10 +131,7 @@ def delete_section(request, section_id, success_url=None):
             else:
                 success_url = "/"
         return HttpResponseRedirect(success_url)
-    return HttpResponse("""
-<html><body><form action="." method="post">Are you Sure?
-<input type="submit" value="Yes, delete it" /></form></body></html>
-""")
+    return TemplateResponse(request, 'delete_confirm.html')
 
 
 def move_section(request, section_id, success_url=None):
@@ -135,7 +140,7 @@ def move_section(request, section_id, success_url=None):
     section = get_object_or_404(Section, id=section_id)
     section.save_version(request.user, "move section")
 
-    form = MoveNodeForm(request.POST, instance=section)
+    form = MoveSectionForm(request.POST, instance=section)
     if form.is_valid():
         to_section = get_object_or_404(
             Section,
@@ -200,8 +205,9 @@ def exporter(request):
         protocol = "https"
     url_base = protocol + "//" + request.get_host()
     for pb in PageBlock.objects.filter(section__hierarchy=h):
-        if hasattr(pb.block(), 'list_resources'):
-            for r in pb.block().list_resources():
+        block = pb.block()
+        if hasattr(block, 'list_resources'):
+            for r in block.list_resources():
                 resources.append(url_base + r)
     data['resources'] = resources
     resp = HttpResponse(dumps(data))
@@ -209,7 +215,6 @@ def exporter(request):
     return resp
 
 
-@render_to("revert_confirm.html")
 def revert_to_version(request, version_id):
     v = get_object_or_404(Version, pk=version_id)
     if request.method == "POST":
@@ -238,4 +243,4 @@ def revert_to_version(request, version_id):
         v.section.clear_tree_cache()
         return HttpResponseRedirect(v.section.get_edit_url())
     else:
-        return dict(version=v)
+        return render(request, "revert_confirm.html", dict(version=v))
